<?php
/**
*
* @package phpBB Arcade
* @version $Id$
* @author 2011-2017 KillBill - killbill@jatek-vilag.com
* @copyright (c) 2014-2017 https://jv-arcade.com/ - support@jv-arcade.com
* @license http://opensource.org/licenses/gpl-2.0.php GNU General Public License v2
*
*/

namespace jv\arcade\acp;

class main_module
{
	public $u_action;
	public $tpl_name;
	public $page_title;

	protected $admin_path;
	protected $db, $cache, $config, $auth, $request, $user, $template, $root_path, $php_ext;
	protected $arcade_cache, $arcade_config, $arcade;

	public function __construct()
	{
		global $phpbb_admin_path;
		global $db, $cache, $config, $auth, $request, $user, $template, $phpbb_root_path, $phpEx;
		global $arcade_cache, $arcade_config, $arcade;

		define('IN_PHPBB_ARCADE', true);
		$arcade->auth_check(($user->data['user_type'] == USER_FOUNDER) ? 'acp_main' : '');
		$arcade = $arcade->container('admin', true);

		$this->admin_path = $phpbb_admin_path;
		$this->db = $db;
		$this->cache = $cache;
		$this->config = $config;
		$this->auth = $auth;
		$this->request = $request;
		$this->user = $user;
		$this->template = $template;
		$this->root_path = $phpbb_root_path;
		$this->php_ext = $phpEx;
		$this->arcade_cache = $arcade_cache;
		$this->arcade_config = $arcade_config;
		$this->arcade = $arcade;
	}

	public function main($id, $mode)
	{
		// If the online check does not work then comment out the following line.
		define('IN_JV_ARCADE_ONLINE_CHECK', true);

		$notice = array();
		$this->tpl_name = 'arcade/acp_main';
		$user_founder = ($this->user->data['user_type'] == USER_FOUNDER) ? true : false;
		$has_activation_field = $mode_title_disable = false;
		$board3_portal = ($this->arcade->portal->data['show'] && $this->arcade->portal->data['type'] == ARCADE_BOARD3_PORTAL && $this->arcade->ext_enable('jv/arcade_b3portal') && defined('ARCADE_BOARD3_PORTAL_VERSION')) ? true : false;

		if ($this->auth->acl_get('a_arcade_install'))
		{
			$b3 = ($board3_portal && $this->request->variable('b3', false)) ? true : false;

			if ($this->request->variable('version_check', false))
			{
				$mode = 'version_check';
			}

			if ($user_founder)
			{
				if (!$this->arcade_config['activation_key'] && $this->arcade->is_set_get('temp_activation') && !$this->arcade->check_online_web())
				{
					$this->phpbb_arcade_activate();
					trigger_error($this->user->lang['ACP_PHPBB_ARCADE_ACTIVATION_KEY_ACTIVATED'] . adm_back_link($this->u_action), E_USER_NOTICE);
				}

				$has_activation_field = (!$this->arcade->localhost()) ? true : false;

				if ($this->arcade->is_set_get('hide_activation'))
				{
					$hide_activation = ($this->request->variable('hide_activation', false)) ? 1 : 0;
					$this->arcade_config->set('activation_hide', $hide_activation);
					$this->arcade_config['activation_hide'] = $hide_activation;
				}

				if ($this->arcade->is_set_get('hide_b3arcade'))
				{
					$hide_b3arcade = ($this->request->variable('hide_b3arcade', false)) ? 1 : 0;
					$this->arcade_config->set('hide_b3arcade', $hide_b3arcade);
					$this->arcade_config['hide_b3arcade'] = $hide_b3arcade;
				}
			}

			if ($this->request->variable('update_vc', false))
			{
				$arcade_vc = $this->request->variable('arcade_vc', false);
				$this->arcade_config->set('version_check_enable', ($arcade_vc) ? 1 : 0);
				$this->arcade->add_log('admin', 'LOG_ARCADE_VERSION_CHECK' . (($arcade_vc) ? '_ENABLED' : '_DISABLED'));

				trigger_error($this->user->lang['CONFIG_UPDATED'] . (($this->request->is_ajax()) ? '' : adm_back_link($this->u_action)), E_USER_NOTICE);
			}
		}

		if ($board3_portal && $this->request->is_set_post('disable_b3_arcade_modules'))
		{
			if (!$this->arcade_config['hide_b3arcade'] || $user_founder)
			{
				$b3portal_disable = ($this->request->variable('disable_b3_arcade_modules', false)) ? 1 : 0;
				$this->arcade_config->set('b3portal_disable', $b3portal_disable);

				$b3portal_ignstats = ($this->request->variable('b3portal_ignstats', false)) ? 1 : 0;
				$this->arcade_config->set('b3portal_ignstats', $b3portal_ignstats);

				$this->arcade->add_log('admin', 'LOG_B3PORTAL_ARCADE_SETTINGS');

				trigger_error($this->user->lang['CONFIG_UPDATED'] . (($this->request->is_ajax()) ? '' : adm_back_link($this->u_action)), E_USER_NOTICE);
			}
			else
			{
				trigger_error($this->user->lang['NOT_AUTHORISED'], E_USER_WARNING);
			}
		}

		if ($this->auth->acl_get('a_arcade') && $this->auth->acl_get('a_arcade_user') && $this->request->variable('users_default_settings', false))
		{
			$mode = 'users_default_settings';
		}

		switch ($mode)
		{
			case 'main':
				$msg = $this->request->variable('msg', '');
				$action = $this->request->variable('action', '');
				$this->page_title = 'ACP_ARCADE_MAIN';

				if (($action == 'term_of_use' && $this->auth->acl_get('a_arcade_install')) || ($action == 'privacy_policy' && $user_founder))
				{
					$mode_title_disable = true;
					$this->page_title = 'ACP_ARCADE_' . strtoupper($action);

					$back = '';
					if ($action == 'term_of_use')
					{
						$s_content_flow_begin = ($this->user->lang['DIRECTION'] == 'ltr') ? 'left' : 'right';
						$back = '<br /><br /><a href="' . $this->u_action . '" style="float: ' . $s_content_flow_begin . ';">&laquo; ' . $this->user->lang['BACK'] . '</a>';
					}

					$this->template->assign_vars(array(
						'S_ARCADE_USE_ONLY_DESC'	=> true,
						'L_ARCADE_TITLE'			=> $this->user->lang[$this->page_title],
						'L_ARCADE_TITLE_EXPLAIN'	=> sprintf($this->user->lang['ACP_ARCADE_' . strtoupper($action) . '_EXPLAIN'], $this->arcade_config['support_domain']) . $back
					));

					break;
				}

				if ($has_activation_field && $msg)
				{
					$rkey = $this->request->variable('rkey', '');
					$extime = $this->request->variable('extime', '');

					switch ($msg)
					{
						case 'ok':
							// Attention! The manual database manipulation is not worth anything! The key plays only a secondary role.
							$this->phpbb_arcade_activate($rkey, $extime);

							trigger_error($this->user->lang['ACP_PHPBB_ARCADE_ACTIVATION_KEY_ACTIVATED'] . adm_back_link($this->u_action), E_USER_NOTICE);
						break;

						case 'active':
							$msg = $this->user->lang['ACP_PHPBB_ARCADE_ACTIVATION_KEY_ALREADY_ACTIVATED'];

							if (!$this->arcade_config['activation_key'])
							{
								$activation_time = (!$this->arcade_config['activation_start']) ? true : false;
								$this->phpbb_arcade_activate($rkey, $extime, $activation_time);
							}
						break;

						case 'invalid':
							$msg = $this->user->lang['ACP_PHPBB_ARCADE_ACTIVATION_KEY_INVALID'];
						break;

						case 'version':
							$msg = $this->user->lang['ACP_PHPBB_ARCADE_ACTIVATION_KEY_INVALID_VERSION'];
						break;

						case 'blocked':
							$msg = sprintf($this->user->lang['ACP_PHPBB_ARCADE_ACTIVATION_BLOCKED'], '<a href="mailto:support@jv-arcade.com">', '</a>');
						break;
					}

					trigger_error($msg . adm_back_link($this->u_action), E_USER_WARNING);
				}

				if ($action)
				{
					if (!confirm_box(true))
					{
						switch ($action)
						{
							case 'reset_arcade':
							case 'reset_scores_all':
							case 'reset_playing_record':
							case 'reset_challenge':
							case 'reset_comment':
							case 'reset_users_data':
							case 'reset_users_settings':
							case 'reset_downloads':
							case 'reset_jackpot':
							case 'reset_points':
							case 'reset_super_champion':
							case 'purge_sessions':
							case 'resync_totals_data':
							case 'resync_users_total_data':
								$confirm = true;
								$confirm_lang = 'ARCADE_' . strtoupper($action);
							break;

							default:
								$confirm = false;
							break;
						}

						if ($confirm)
						{
							$s_hidden_fields = array(
								'i'			=> $id,
								'mode'		=> $mode,
								'action'	=> $action
							);

							if ($action == 'reset_arcade' || $action == 'reset_scores_all')
							{
								$this->arcade->confirm_box(($action == 'reset_arcade') ? 'retain_super_champion' : 'retain_plays', $confirm_lang, $s_hidden_fields);
							}
							else
							{
								confirm_box(false, $confirm_lang, build_hidden_fields($s_hidden_fields), 'confirm_body.html');
							}
						}
					}
					else
					{
						if ((!$this->auth->acl_get('a_arcade')) ||
							($action == 'reset_points' && !$this->auth->acl_get('a_arcade_points_settings')) ||
							(in_array($action, array('reset_scores_all', 'reset_users_data', 'reset_users_settings', 'reset_comment', 'reset_super_champion')) && !$this->auth->acl_get('a_arcade_user')) ||
							(in_array($action, array('reset_arcade', 'reset_challenge')) && !$this->auth->acl_gets('a_arcade_game', 'a_arcade_user')) ||
							($action == 'purge_sessions' && !$user_founder))
						{
							send_status_line(403, 'Forbidden');
							trigger_error($this->user->lang['NO_AUTH_OPERATION'] . (($this->request->is_ajax()) ? '' : adm_back_link($this->u_action)), E_USER_WARNING);
						}

						switch ($action)
						{
							case 'reset_arcade':
							case 'reset_scores_all':
							case 'reset_playing_record':
							case 'reset_challenge':
							case 'reset_comment':
							case 'reset_users_data':
							case 'reset_users_settings':
							case 'reset_downloads':
							case 'reset_jackpot':
							case 'reset_points':
							case 'reset_super_champion':
							case 'purge_sessions':
								$this->reset($action);
							break;

							case 'resync_totals_data':
								$this->arcade->sync('total_data', 'all');
								$this->cache->destroy('_arcade_games_filesize');
							break;

							case 'resync_users_total_data':
								$this->arcade->sync('users_total_data');
								$this->arcade->cache_purge('score');
							break;

							default:
								trigger_error('NO_MODE', E_USER_ERROR);
							break;
						}

						$log_msg = 'LOG_ARCADE_' . strtoupper($action);

						if (in_array($action, array('reset_arcade', 'reset_scores_all')))
						{
							$var_key = ($action == 'reset_arcade') ? 'retain_super_champion' : 'retain_plays';
							$this->arcade->add_rvlog($log_msg, $var_key);
						}
						else
						{
							$this->arcade->add_log('admin', $log_msg);
						}

						trigger_error($this->user->lang['ARCADE_' . strtoupper($action) . '_DONE'] . (($this->request->is_ajax()) ? '' : adm_back_link($this->u_action)), E_USER_NOTICE);
					}
				}

				$latest_version_info = $update_to_date = false;
				$versioncheck_force = ($this->auth->acl_get('a_arcade_install') && $this->request->variable('versioncheck_force', false)) ? true : false;

				if ($this->arcade_config['version_check_enable'] || $versioncheck_force)
				{
					if (($latest_version_info = $this->vc()) === false)
					{
						$this->template->assign_var('S_VERSIONCHECK_FAIL', true);
					}
					else
					{
						$latest_version_info = explode("\n", $latest_version_info);
						$update_to_date = phpbb_version_compare($this->arcade_config['version'], trim($latest_version_info[0]), '<') ? false : true;
					}
				}
				else
				{
					$update_to_date = true;
				}

				if ($board3_portal)
				{
					$this->template->assign_vars(array(
						'S_B3PORTAL_ARCADE'	=> true,
						'S_HIDE_B3ARCADE'	=> ($this->arcade_config['hide_b3arcade']) ? true : false,

						'U_HIDE_B3ARCADE'	=> $this->u_action . '&amp;hide_b3arcade=1',
						'U_SHOW_B3ARCADE'	=> $this->u_action . '&amp;hide_b3arcade=0'
					));

					if (!$this->arcade_config['hide_b3arcade'])
					{
						$pam_update_to_date = false;

						if (!empty($latest_version_info[10]))
						{
							$pam_update_to_date = phpbb_version_compare(ARCADE_BOARD3_PORTAL_VERSION, trim($latest_version_info[10]), '<') ? false : true;
						}

						$b3arcade_version = ((($this->auth->acl_get('a_arcade_install') && $this->arcade_config['version_check_enable']) || ($versioncheck_force && $b3)) ? '<strong style="color:#'. ($pam_update_to_date ? '228822' : 'BC2A4D') .'">' . ARCADE_BOARD3_PORTAL_VERSION : '<strong>' . ARCADE_BOARD3_PORTAL_VERSION) . '</strong>';

						$this->template->assign_vars(array(
							'S_DISABLE_ALL_B3MODULES'			=> ($this->arcade_config['b3portal_disable']) ? true : false,
							'S_B3MODULES_IGN_STATS_AUTH'		=> ($this->arcade_config['b3portal_ignstats']) ? true : false,
							'S_B3PORTAL_ARCADE_NOT_COMPATIBLE'	=> (!$this->arcade->access('portal', true, true)) ? $this->user->lang['B3PORTAL_ARCADE_MODULES_NOT_COMPATIBLE'] : false,

							'B3PORTAL_ARCADE_VERSION'			=> ($this->auth->acl_get('a_arcade_install')) ? '<a href="' . $this->u_action . '&amp;version_check=1&amp;b3=1' . '" title="' . $this->user->lang['MORE_INFORMATION'] . '">' . $b3arcade_version . '</a>&nbsp;[&nbsp;<a href="' . $this->u_action . '&amp;versioncheck_force=1&amp;b3=1' . '">' . $this->user->lang['VERSIONCHECK_FORCE_UPDATE'] . '</a>&nbsp;]' : $b3arcade_version
						));
					}
				}

				$s_hidden_fields = '';
				$arcade_version = ((($this->auth->acl_get('a_arcade_install') && $this->arcade_config['version_check_enable']) || ($versioncheck_force && !$b3)) ? '<strong style="color:#'. (($update_to_date) ? '228822' : 'BC2A4D') .';">' . $this->arcade_config['version'] : '<strong>' . $this->arcade_config['version']) . '</strong>';

				if ($has_activation_field)
				{
					$new_activation_key = $this->request->variable('new_activation_key', false);

					if (!$this->arcade_config['activation_key'] && !$new_activation_key)
					{
						$notice[] = $this->user->lang['ACP_PHPBB_ARCADE_NOT_ACTIVATED'];
					}

					$display_activation_field = true;
					$activation = (!$this->arcade_config['activation_key'] || $new_activation_key) ? true : false;

					if ($activation)
					{
						if ($this->request->variable('diff', false) && $this->arcade_config['activation_key'])
						{
							$this->arcade_config->set('activation_key', '');
						}

						if (!defined('IN_JV_ARCADE_ONLINE_CHECK') || $this->arcade->check_online_web())
						{
							$s_hidden_fields = build_hidden_fields(array(
								'lang'				=> $this->user->data['user_lang'],
								'arcade_version'	=> $this->arcade_config['version'],
								'web_return_url'	=> base64_encode(generate_board_url() . '/adm/' . str_replace($this->admin_path, '', $this->u_action)),
								'username'			=> $this->user->data['username'],
								'useremail'			=> $this->user->data['user_email'],
								'userip'			=> $this->user->ip,
								'phpbb_version'		=> $this->config['version'],
								'newkey'			=> ($this->arcade_config['activation_key']) ? $this->arcade_config['activation_key'] : 0
							));
						}
						else
						{
							$activation = false;
							$display_activation_field = ($new_activation_key || !$this->arcade_config['activation_key']) ? false : true;
							$notice[] = sprintf($this->user->lang['ACP_PHPBB_ARCADE_SITE_NOT_AVAILABLE'], $this->arcade_config['connect_domain']) . ((!$this->arcade_config['activation_key']) ? '<br />' . sprintf($this->user->lang['ACP_PHPBB_ARCADE_SITE_NOT_AVAILABLE_ACTIVATE_TEST'], $this->arcade_config['connect_domain'], '<a href="' . $this->u_action . '&amp;temp_activation=1' . '">', '</a>') : '');
						}
					}

					if ($display_activation_field)
					{
						$activation_key = ($this->arcade_config['activation_key'] == 'test') ? $this->user->lang['ARCADE_TEST'] : (($this->arcade_config['activation_key'] == 'free') ? $this->user->lang['ARCADE_FREE'] : $this->arcade_config['activation_key']);

						$this->template->assign_vars(array(
							'S_HAS_ARCADE_ACTIVATION'	=> true,
							'S_ARCADE_NEW_ACTIVE'		=> $activation,
							'S_ACTIVATION_HIDE'			=> ($this->arcade_config['activation_hide']) ? true : false,
							'S_ACTIVATE_KEY'			=> $this->arcade_config['activation_key'],
							'S_KEY_END'					=> ($this->arcade_config['activation_end'] && $this->arcade_config['activation_end'] < time()) ? true : false,
							'S_ACTIVATE_KEY_PRIVATE'	=> ($this->arcade_config['activation_key'] && !in_array($this->arcade_config['activation_key'], array('test', 'free'))) ? true : false,

							'U_HIDE_ACTIVATION'			=> $this->u_action . '&amp;hide_activation=1',
							'U_SHOW_ACTIVATION'			=> $this->u_action . '&amp;hide_activation=0',

							'ACTIVATE_KEY'				=> $activation_key,
							'ACTIVATION_KEY_EXPLAIN'	=> sprintf($this->user->lang['ACP_PHPBB_ARCADE_ACTIVATION_KEY_EXPLAIN'], '<a onclick="window.open(this.href); return false;" href="'. $this->u_action . '&amp;action=privacy_policy' .'">', '</a>'),
							'ARCADE_ACTIVATION_DATE'	=> $this->user->format_date($this->arcade_config['activation_start']),
							'ARCADE_EXPIRATION_DATE'	=> ($this->arcade_config['activation_end']) ? $this->user->format_date($this->arcade_config['activation_end'], false, true) : $this->user->lang['ARCADE_NONE']
						));
					}
				}
				else if (!$this->arcade_config['activation_key'] && $this->arcade->localhost())
				{
					$this->phpbb_arcade_activate();
				}

				$install_verify = '';
				if ($this->auth->acl_get('a_arcade_install'))
				{
					$verify_link	= append_sid("{$this->admin_path}index.{$this->php_ext}", $this->arcade->module_url('install') . '&amp;mode=install_verify');
					$install_verify = '[ <a href="'. $verify_link .'" title="'. $this->user->lang['ACP_ARCADE'] .' - '. $this->user->lang['ACP_ARCADE_INSTALL_VERIFY'] .'">'. $this->user->lang['ACP_ARCADE_INSTALL_VERIFY'] .'</a> ]';
				}

				$this->template->assign_vars(array(
					'S_VERSION_UP_TO_DATE'				=> $update_to_date,
					'S_USER_FOUNDER'					=> $user_founder,
					'S_ARCADE_INSTALL'					=> $this->auth->acl_get('a_arcade_install'),
					'S_ARCADE_VC'						=> ($this->arcade_config['version_check_enable']) ? true : false,
					'S_ARCADE_MANAGE'					=> ($this->auth->acl_get('a_arcade')) ? true : false,
					'S_GAMES_USERS_MANAGE'				=> ($this->auth->acl_gets('a_arcade_game', 'a_arcade_user')) ? true : false,
					'S_USERS_MANAGE'					=> ($this->auth->acl_get('a_arcade_user')) ? true : false,
					'S_SHOW_POINTS'						=> ($this->arcade->points->data['installed'] && $this->auth->acl_get('a_arcade_points_settings')) ? true : false,
					'S_HIDDEN_FIELDS'					=> $s_hidden_fields,

					'U_ACTION'							=> $this->u_action,
					'U_VERSIONCHECK'					=> $this->u_action . '&amp;version_check=1',
					'U_VERSIONCHECK_FORCE'				=> $this->u_action . '&amp;versioncheck_force=1',
					'U_USERS_DEFAULT_SETTINGS'			=> $this->u_action . '&amp;users_default_settings=1',

					'ACP_ARCADE_TERM_OF_USE'			=> '[ <a href="'. $this->u_action . '&amp;action=term_of_use' .'">'. $this->user->lang['ACP_ARCADE_TERM_OF_USE'] .'</a> ]',
					'ARCADE_INSTALL_VERIFY'				=> $install_verify,
					'ARCADE_START'						=> $this->user->format_date($this->arcade_config['arcade_startdate']),
					'STAT_INSTALL_GAMES'				=> $this->arcade->number_format(intval($this->arcade_config['install_games'])),
					'STAT_DOWNLOAD_GAMES'				=> $this->arcade->number_format(intval($this->arcade_config['total_downloads'])),
					'STAT_DOWNLOAD_GAMES_DAY'			=> $this->data_per_day($this->arcade_config['total_downloads']),
					'STAT_PLAYS_GAMES'					=> $this->arcade->number_format(intval($this->arcade_config['total_plays'])),
					'STAT_PLAYS_GAMES_DAY'				=> $this->data_per_day($this->arcade_config['total_plays']),
					'CHALLENGE_STAT_PLAYS_GAMES'		=> $this->arcade->number_format(intval($this->arcade_config['challenge_total_plays'])),
					'CHALLENGE_STAT_PLAYS_GAMES_DAY'	=> $this->data_per_day($this->arcade_config['challenge_total_plays'], 'challenge'),
					'STAT_USERS_PLAYED'					=> $this->arcade->number_format(intval($this->arcade->get->total('user', 'played_users'))),
					'CHALLENGE_STAT_USERS_PLAYED'		=> $this->arcade->number_format(intval($this->arcade->get->total('user', 'challenge_played_users'))),
					'ARCADE_RESETDATE'					=> $this->arcade_config['arcade_resetdate'] ? $this->user->format_date($this->arcade_config['arcade_resetdate']) : $this->user->lang['ACP_ARCADE_RESET_NO_DATE'],
					'CHALLENGE_RESETDATE'				=> $this->user->format_date($this->arcade_config['challenge_resetdate']),
					'STATS_FINISHED_TOURS'				=> (!empty($this->arcade_config['tour_total'])) ? $this->arcade->number_format(intval($this->arcade_config['tour_total'])) : 0,
					'STATS_TOUR_USERS_PLAYED'			=> (!empty($this->arcade_config['tour_total'])) ? $this->arcade->number_format(intval($this->arcade->get->total('user', 'tour_played_users'))) : 0,
					'ARCADE_VERSION'					=> ($this->auth->acl_get('a_arcade_install')) ? '<a href="' . $this->u_action . '&amp;version_check=1' . '" title="' . $this->user->lang['MORE_INFORMATION'] . '">' . $arcade_version . '</a>&nbsp;[&nbsp;<a href="' . $this->u_action . '&amp;versioncheck_force=1' . '">' . $this->user->lang['VERSIONCHECK_FORCE_UPDATE'] . '</a>&nbsp;]' : $arcade_version,
					'ARCADE_GAMES_FILESIZE'				=> get_formatted_filesize($this->arcade_cache->obtain_arcade_games_filesize()),

					'L_ARCADE_FINISHED_TOURS'			=> $this->user->lang['ARCADE_FINISHED_TOUR' . (($this->arcade_config['tour_total'] > 1) ? 'S' : '')]
				));

				if ($this->config['cookie_secure'])
				{
					$cookie_set = ($this->auth->acl_get('a_server')) ? '<br />' . sprintf($this->user->lang['ARCADE_COOKIE_SECURE_LINK'], '<a href="' . append_sid("{$this->admin_path}index.{$this->php_ext}", 'i=board&amp;mode=cookie') . '">', '</a>') : '';

					$notice[] = $this->user->lang['ARCADE_COOKIE_SECURE_NOTICE'] . $cookie_set;
				}

				if (sizeof($notice))
				{
					$this->template->assign_vars(array(
						'S_ARCADE_NOTICE'	=> true,

						'ARCADE_NOTICE'		=> implode('<br /><br />', $notice)
					));
				}
			break;

			case 'version_check':
				$this->page_title = 'ACP_VERSION_CHECK';

				$info = $this->vc(true);
				$info = explode("\n", $info);

				if ($info === false || (!$b3 && empty($info[0])) || ($b3 && empty($info[10])))
				{
					trigger_error('VERSIONCHECK_FAIL', E_USER_WARNING);
				}

				$current_version  = ($b3) ? ARCADE_BOARD3_PORTAL_VERSION : $this->arcade_config['version'];
				$latest_version   = trim(($b3) ? $info[10] : $info[0]);
				$announcement_url = ($b3) ? ((!empty($info[11])) ? trim($info[11]) : '') : ((!empty($info[1])) ? trim($info[1]) : '');
				$announcement_url = (strpos($announcement_url, '&amp;') === false) ? str_replace('&', '&amp;', $announcement_url) : $announcement_url;

				if ($b3)
				{
					$mode = 'b3version_check';
					$up_to_date			= (phpbb_version_compare(ARCADE_BOARD3_PORTAL_VERSION, $latest_version, '<')) ? false : true;
					$not_up_date		= ($announcement_url) ? sprintf($this->user->lang['B3PORTAL_ARCADE_VERSION_NOT_UP_TO_DATE_ACP'], '<a onclick="window.open(this.href); return false;" href="' . $announcement_url . '">', '</a>') : '';
				}
				else
				{
					$update_link = append_sid($this->root_path . 'install/index.' . $this->php_ext);

					$up_to_date = (phpbb_version_compare($this->arcade_config['version'], $latest_version, '<')) ? false : true;
					$not_up_date = $this->user->lang['ARCADE_VERSION_NOT_UP_TO_DATE_ACP'];

					$next_feature_version = $next_feature_announcement_url = false;
					if (!empty($info[2]))
					{
						$next_feature_version = trim($info[2]);
						$next_feature_announcement_url = trim($info[3]);
					}

					$this->template->assign_vars(array(
						'NEXT_FEATURE_VERSION'	=> $next_feature_version,
						'UPDATE_INSTRUCTIONS'	=> ($announcement_url) ? sprintf($this->user->lang['ARCADE_UPDATE_INSTRUCTIONS'], $announcement_url, $update_link) : '',
						'UPGRADE_INSTRUCTIONS'	=> $next_feature_version ? sprintf($this->user->lang['ARCADE_UPGRADE_INSTRUCTIONS'], $next_feature_version, $next_feature_announcement_url) : false
					));
				}

				$this->template->assign_vars(array(
					'S_VERSION_CHECK'			=> true,
					'S_UP_TO_DATE'				=> $up_to_date,
					'U_VERSIONCHECK_FORCE'		=> $this->u_action . '&amp;version_check=1&amp;versioncheck_force=1' . (($b3) ? '&amp;b3=1' : ''),

					'ARCADE_UP_TO_DATE_ACP'		=> $this->user->lang[(($b3) ? 'B3PORTAL_' : '') . 'ARCADE_VERSION_UP_TO_DATE_ACP'],
					'ARCADE_NOT_UP_TO_DATE'		=> $this->user->lang[(($b3) ? 'B3PORTAL_' : '') . 'ARCADE_VERSION_NOT_UP_TO_DATE_TITLE'],
					'ARCADE_NOT_UP_TO_DATE_ACP'	=> ($not_up_date) ? $not_up_date . '<br />' :  '',

					'LATEST_VERSION'			=> '<strong style="color:#228822">' . $latest_version . '</strong>',
					'CURRENT_VERSION'			=> '<strong style="color:#'. ($up_to_date ? '228822' : 'BC2A4D') .'">' . $current_version . '</strong>'
				));
			break;

			case 'users_default_settings':
				$this->page_title = 'ACP_ARCADE_USERS_DEFAULT_SETTINGS_TITLE';

				if ($this->arcade->is_post_empty('update'))
				{
					$cat_style_height = (int) $this->request->variable('cat_style_height', 0);

					if ($cat_style_height < 0 || $cat_style_height > 30)
					{
						$cat_style_height = 0;
					}

					$this->arcade_config->set('default_cat_style', (($this->request->variable('cat_style', false)) ? 1 : 0));
					$this->arcade_config->set('override_user_cat_style', (($this->request->variable('override_cat_style', false)) ? 1 : 0));
					$this->arcade_config->set('cat_style_height', $cat_style_height);
					$this->arcade_config->set('default_games_sort_order', $this->request->variable('games_sort_order', 'n'));
					$this->arcade_config->set('default_games_sort_dir', $this->request->variable('games_sort_dir', 'a'));

					$this->arcade->add_log('admin', 'LOG_ARCADE_USERS_DEFAULT_SETTINGS');
					trigger_error($this->user->lang['CONFIG_UPDATED'] . adm_back_link($this->u_action . '&amp;users_default_settings=1'), E_USER_NOTICE);
				}

				$this->user->add_lang('acp/board');
				$this->user->add_lang_ext('jv/arcade', 'info_ucp_arcade');

				$s_hidden_fields = build_hidden_fields(array(
					'users_default_settings' => 1
				));

				$this->template->assign_vars(array(
					'S_USERS_DEFAULT_SETTINGS'	=> true,
					'S_OVERRIDE_STYLE'			=> ($this->arcade_config['override_user_cat_style']) ? true : false,
					'S_HIDDEN_FIELDS'			=> $s_hidden_fields,

					'CAT_STYLES'				=> $this->arcade->select_cat_styles($this->arcade_config['default_cat_style']),
					'CAT_STYLE_HEIGHT'			=> (int) $this->arcade_config['cat_style_height'],
					'GAMES_SORT_ORDER'			=> $this->arcade->select_games_sort_order($this->arcade_config['default_games_sort_order']),
					'GAMES_SORT_DIR'			=> $this->arcade->select_games_sort_dir($this->arcade_config['default_games_sort_dir'])
				));
			break;

			default:
				trigger_error('NO_MODE', E_USER_ERROR);
			break;
		}

		if (!$mode_title_disable)
		{
			$this->template->assign_vars(array(
				'L_ARCADE_TITLE'			=> $this->user->lang['ACP_ARCADE_' . strtoupper($mode) . '_TITLE'],
				'L_ARCADE_TITLE_EXPLAIN'	=> $this->user->lang['ACP_ARCADE_' . strtoupper($mode) . '_TITLE_EXPLAIN']
			));
		}
	}

	function reset($action)
	{
		switch ($action)
		{
			case 'reset_arcade':
				$retain_super_champion = ($this->request->variable('retain_super_champion', false)) ? true : false;

				$delete_tables = array(ARCADE_FAVS_TABLE, ARCADE_RATING_TABLE, ARCADE_DOWNLOAD_TABLE);

				if (!$retain_super_champion)
				{
					$delete_tables[] = ARCADE_SUPER_SCORES_TABLE;
				}

				$this->arcade->delete_table_data($delete_tables);

				$sql_ary = array(
					'game_votetotal'		=> 0,
					'game_download_total'	=> 0,
					'game_votesum'			=> 0,
					'game_download_cost'	=> 0,
					'game_cost'				=> 0,
					'game_reward'			=> 0,
					'game_jackpot'			=> (float) $this->arcade_config['jackpot_minimum'],
					'game_use_jackpot'		=> 0,
				);

				$this->arcade_config->set('total_downloads', 0, true);

			case 'reset_playing_record':
				$this->arcade_config->set('record_playing_users', 0, true);
				$this->arcade_config->set('record_playing_date', time(), true);

				if ($action == 'reset_playing_record')
				{
					break;
				}
			case 'reset_scores_all':
				$retain_plays = ($action == 'reset_scores_all' && $this->request->variable('retain_plays', false)) ? true : false;

				$delete_tables = array(ARCADE_SCORES_TABLE);

				if (!$retain_plays)
				{
					$delete_tables[] = ARCADE_PLAYS_TABLE;
				}

				$this->arcade->delete_table_data($delete_tables);

				if (!$retain_plays)
				{
					$sql_ary['game_plays'] = 0;
				}

				$sql_ary['game_highscore']	= 0;
				$sql_ary['game_highuser']	= 0;
				$sql_ary['game_highdate']	= 0;

				$sql = 'UPDATE ' . ARCADE_GAMES_TABLE. '
						SET ' . $this->db->sql_build_array('UPDATE', $sql_ary);
				$this->db->sql_query($sql);

				$sql_ary = array(
					'cat_last_play_game_id'			=> 0,
					'cat_last_play_game_name'		=> '',
					'cat_last_play_user_id'			=> 0,
					'cat_last_play_score'			=> 0,
					'cat_last_play_time'			=> 0,
					'cat_last_play_username'		=> '',
					'cat_last_play_user_colour'		=> '',
				);

				if (!$retain_plays)
				{
					$sql_ary['cat_plays'] = 0;
				}

				if ($action == 'reset_arcade')
				{
					$sql_ary['cat_download_cost'] = 0;
					$sql_ary['cat_cost'] = 0;
					$sql_ary['cat_reward'] = 0;
					$sql_ary['cat_use_jackpot'] = 0;
				}

				$sql = 'UPDATE ' . ARCADE_CATS_TABLE . '
						SET ' . $this->db->sql_build_array('UPDATE', $sql_ary);
				$this->db->sql_query($sql);

				$this->arcade->sync('category');

				if (!$retain_plays)
				{
					$this->arcade_config->set('total_plays', 0, true);
					$this->arcade_config->set('total_plays_time', 0, true);
					$this->arcade_config->set('arcade_resetdate', time());
				}

				$sql = "UPDATE " . ARCADE_USERS_TABLE . '
						SET arcade_total_wins = 0';

				if ($action == 'reset_arcade')
				{
					$sql .= ', games_last_search = 0, games_last_search_term = "", user_arcade_last_download = 0, arcade_total_downloads = 0';

					if (!$retain_super_champion)
					{
						$sql .= ', arcade_total_super_scores = 0';
					}
				}

				if (!$retain_plays)
				{
					$sql .= ', user_arcade_last_play = 0, arcade_total_played = 0, arcade_total_plays = 0, arcade_total_time = 0';
				}

				$this->db->sql_query($sql);

				$this->arcade->cache_purge();
			break;

			case 'reset_users_data':
				$this->arcade->delete_table_data(array(ARCADE_FAVS_TABLE, ARCADE_RATING_TABLE));

				$sql = 'UPDATE ' . ARCADE_GAMES_TABLE . '
						SET game_votetotal = 0, game_votesum = 0';
				$this->db->sql_query($sql);
			break;

			case 'reset_users_settings':
				$sql = 'UPDATE ' . ARCADE_USERS_TABLE . '
						SET ' . $this->db->sql_build_array('UPDATE', $this->arcade->user_default_settings()) . '
						WHERE user_id <> ' . ANONYMOUS;
				$this->db->sql_query($sql);
/*
				// default settings guest user
				$sql = 'UPDATE ' . ARCADE_USERS_TABLE . '
						SET ' . $this->db->sql_build_array('UPDATE', arcade_user_default_settings(ANONYMOUS)) . '
						WHERE user_id = ' . ANONYMOUS;
				$this->db->sql_query($sql);
*/
			break;

			case 'reset_downloads':
				$this->arcade->delete_table_data(ARCADE_DOWNLOAD_TABLE);

				$sql = 'UPDATE ' . ARCADE_GAMES_TABLE . '
						SET game_download_total = 0';
				$this->db->sql_query($sql);

				$sql = 'UPDATE ' . ARCADE_USERS_TABLE . '
						SET user_arcade_last_download = 0, arcade_total_downloads = 0';
				$this->db->sql_query($sql);

				$this->arcade_config->set('total_downloads', 0, true);
			break;

			case 'reset_points':
				$sql_ary = array(
					'cat_download_cost'	=> 0,
					'cat_cost'			=> 0,
					'cat_reward'		=> 0,
					'cat_use_jackpot'	=> 0,
				);

				$sql = 'UPDATE ' . ARCADE_CATS_TABLE . '
					SET ' . $this->db->sql_build_array('UPDATE', $sql_ary);
				$this->db->sql_query($sql);

				$sql_ary = array(
					'game_download_cost'	=> 0,
					'game_cost'				=> 0,
					'game_reward'			=> 0,
					'game_use_jackpot'		=> 0,
				);

				$this->arcade->cache_purge();
			case 'reset_jackpot':
				$sql_ary['game_jackpot'] = (float) $this->arcade_config['jackpot_minimum'];

				$sql = 'UPDATE ' . ARCADE_GAMES_TABLE . '
						SET ' . $this->db->sql_build_array('UPDATE', $sql_ary);
				$this->db->sql_query($sql);
			break;

			case 'purge_sessions':
				$this->arcade->delete_table_data(array(ARCADE_ACCESS_TABLE, ARCADE_SESSIONS_TABLE));
			break;

			case 'reset_challenge':
				$this->arcade->delete_table_data(array(ARCADE_CHALLENGE_TABLE, ARCADE_CHALLENGE_CHAMP_TABLE));

				$this->arcade_config->set('challenge_resetdate', time());
				$this->arcade_config->set('challenge_total_plays', 0, true);
				$this->arcade_config->set('challenge_total_plays_time', 0, true);

				$this->cache->destroy('sql', ARCADE_CHALLENGE_CHAMP_TABLE);

				$this->cache->destroy('_arcade_challenge_leaders');
				$this->cache->destroy('_arcade_challenge_leaders_all');
			break;

			case 'reset_comment':
				$sql_ary = array(
					'comment_text'		=> '',
					'comment_bitfield'	=> '',
					'comment_options'	=> 0,
					'comment_uid'		=> ''
				);

				$sql = 'UPDATE ' . ARCADE_SCORES_TABLE . '
						SET ' . $this->db->sql_build_array('UPDATE', $sql_ary);
				$this->db->sql_query($sql);
			break;

			case 'reset_super_champion':
				$sql = 'SELECT g.game_id
						FROM ' . ARCADE_GAMES_TABLE . ' g
						LEFT JOIN ' . ARCADE_SUPER_SCORES_TABLE . ' s ON g.game_id = s.game_id
						WHERE g.game_highscore <> s.score';
				$result = $this->db->sql_query($sql);
				$game_ids = array();
				while ($row = $this->db->sql_fetchrow($result))
				{
					$game_ids[] = (int) $row['game_id'];
				}
				$this->db->sql_freeresult($result);

				if (sizeof($game_ids))
				{
					$sql = 'DELETE FROM ' . ARCADE_SUPER_SCORES_TABLE . '
							WHERE ' . $this->db->sql_in_set('game_id', $game_ids);
					$this->db->sql_query($sql);

					$this->arcade->sync('super_champ', $game_ids);
					$this->arcade->sync('users_total_data');
					$this->arcade->cache_purge('score');
				}
			break;

			default:
				trigger_error('NO_MODE', E_USER_ERROR);
			break;
		}
	}

	private function phpbb_arcade_activate($key = 'test', $end_time = 0, $activation_time = true)
	{
		$this->arcade_config->set('activation_key', $key);
		$this->arcade_config->set('activation_end', (int) $end_time);

		if ($activation_time)
		{
			$this->arcade_config->set('activation_start', time());
		}
	}

	function data_per_day($data, $type = '')
	{
		$date = intval(($type == 'challenge') ? $this->arcade_config['challenge_resetdate'] : (($this->arcade_config['arcade_resetdate']) ? $this->arcade_config['arcade_resetdate'] : $this->arcade_config['arcade_startdate']));

		$arcadedays = (time() - $date) / 86400;
		$arcadedays = ($arcadedays <= 0) ? 1 : $arcadedays;
		$data_per_day = sprintf('%.2f', $data / $arcadedays);

		if ($data_per_day > $data)
		{
			$data_per_day = sprintf('%.2f', $data);
		}

		return $data_per_day;
	}

	function vc($warn_fail = false)
	{
		return $this->arcade->obtain_version_info('_arcade_vc', str_replace(array('http://', 'https://', '/'), '', $this->arcade_config['connect_domain']), '/ac/v', 'version2.txt', $this->request->variable('versioncheck_force', false), $warn_fail);
	}
}
