<?php
/**
*
* @package phpBB Arcade
* @version $Id$
* @author 2011-2017 KillBill - killbill@jatek-vilag.com
* @copyright (c) 2014-2017 https://jv-arcade.com/ - support@jv-arcade.com
* @license http://opensource.org/licenses/gpl-2.0.php GNU General Public License v2
*
*/

namespace jv\arcade\inc\game;

class install_file
{
	protected $db, $arcade, $file_functions, $root_path, $php_ext;

	public function __construct($db, $arcade, $file_functions, $root_path, $php_ext)
	{
		$this->db = $db;
		$this->arcade = $arcade;
		$this->file_functions = $file_functions;
		$this->root_path = $root_path;
		$this->php_ext = $php_ext;
	}

	public function data($dir, $file, $install_file)
	{
		$if = $game_data = false;

		if (!file_exists($install_file))
		{
			$type = '';
			if ($if = $this->path($dir, $file, $type))
			{
				$install_file = $if;
			}
		}

		if (file_exists($install_file))
		{
			if (!$this->version_check($install_file) || !$this->require_var($game_data, $install_file))
			{
				if (!$this->convert($dir, $file))
				{
					return false;
				}
			}

			@include($install_file);

			if ($game_data === false || !is_array($game_data))
			{
				return false;
			}
		}

		return $game_data;
	}

	public function path($dir, &$file_name, &$type, $extract = false, $re_check = false)
	{
		// Check to make sure a valid config file is present
		$install_file = '';
		$orig_file_name = $file_name;

		// As far as I know all ibpro games start with "game_" if this is not the case
		// please let me know as this will have to be changed
		if (($extract || $re_check) && substr($file_name, 0, 5) == 'game_')
		{
			$file_name = substr($file_name, 5);
		}

		if ($extract)
		{
			$game_config1 = $game_config3 = $dir . $file_name . '.' . $this->php_ext;
			$game_config2 = $dir . $orig_file_name . '.' . $this->php_ext;
			$game_xml1	  = $dir . 'game.xml';
			$game_xml2	  = $dir . 'game.xml';
			$v3_config	  = $dir . $file_name . '.game.' . $this->php_ext;
		}
		else
		{
			$game_config1 = $dir . $file_name . '/' . $file_name . '.' . $this->php_ext;
			$game_config2 = $dir . $orig_file_name . '/'. $orig_file_name . '.' . $this->php_ext;
			$game_config3 = $dir . $orig_file_name . '/'. $file_name . '.' . $this->php_ext;
			$game_xml1	  = $dir . $file_name . '/' . 'game.xml';
			$game_xml2	  = $dir . $orig_file_name . '/' . 'game.xml';
			$v3_config	  = $dir . $file_name . '/'. $file_name . '.game.' . $this->php_ext;
		}

		if (file_exists($game_config1))
		{
			$install_file = $game_config1;
			$type = 'arcade';
		}
		else if (file_exists($game_config2))
		{
			$install_file = $game_config2;
			$type = 'orig';
		}
		else if (file_exists($game_config3))
		{
			$install_file = $game_config3;
			$type = 'mix';
		}
		else if (file_exists($game_xml1))
		{
			$install_file = $game_xml1;
			$type = 'xml1';
		}
		else if (file_exists($game_xml2))
		{
			$install_file = $game_xml2;
			$type = 'xml2';
		}
		else if (file_exists($v3_config))
		{
			$install_file = $v3_config;
			$type = 'v3';
		}

		// Improperly packed game checking
		if ($re_check && $install_file)
		{
			if (in_array($type, array('orig', 'mix')))
			{
				$this->file_functions->append_slash($orig_file_name);
				$this->file_functions->move_dir($dir . $orig_file_name, $dir);
				$install_file = str_replace($orig_file_name, '', $install_file);
			}
			else
			{
				$dfn = $file_name;
				$this->file_functions->append_slash($dfn);
				$this->file_functions->move_dir($dir . $dfn, $dir);
				$install_file = str_replace($dfn, '', $install_file);
			}
		}
		else if ($extract && !$re_check && !$install_file)
		{
			// re-check add path + game folder
			$install_file = $this->path($dir, $orig_file_name, $type, false, true);
		}
		// Improperly packed game checking

		return $install_file;
	}

	public function convert($dir, $game_filename, $extract = false)
	{
		$is_xml = $convert_install_file = false;
		$orig_game_filename = $game_filename;

		$type = '';
		$install_file = $this->path($dir, $game_filename, $type, $extract);

		switch ($type)
		{
			case 'orig':
				$game_filename = $orig_game_filename;
			break;

			case 'xml2':
				$game_filename = $orig_game_filename;
			case 'xml1':
				$is_xml = true;
			break;
		}

		if ($install_file)
		{
			$update = true;

			if ($is_xml)
			{
				$install_array = $this->file_functions->xml2array($install_file);
				$this->file_functions->delete_file($install_file);
				$install_file = $dir . (($extract) ? '' : $game_filename . '/') . $game_filename . '.' . $this->php_ext;

				foreach ($install_array as $key => $value)
				{
					if (is_string($value))
					{
						$install_array[$key] = trim($value);
					}
				}

				$data = array(
					'game_name'			=> $install_array['game']['name'],
					'game_desc'			=> $install_array['game']['description'] . ' ' . $install_array['game']['directions'],
					'game_control'		=> UNKNOWN_GAME,
					'game_control_desc'	=> '',
					'game_width'		=> (int) $install_array['game']['width'],
					'game_height'		=> (int) $install_array['game']['height'],
					'game_scoretype'	=> (strtolower($install_array['game']['revscore']) == 'false') ? SCORETYPE_HIGH : SCORETYPE_LOW,
					'game_type'			=> (strtolower($install_array['game']['highscore']) == 'false') ? NOSCORE_GAME : AR_GAME
				);

				$convert_install_file = true;
				$game_filename = $this->file_functions->remove_extension($install_array['game']['swf']);
			}
			else
			{
				$file_contents = file_get_contents($install_file);

				if (strpos($file_contents, 'print_cp_header("v3') !== false)
				{
					$_POST['do'] = 'inactive';

					$search = array('define(', 'error_reporting', 'require', 'include', 'print_cp_header');
					$replace = array('//define(', '//error_reporting', '//require', '//include', '//print_cp_header');
					$this->file_functions->delete_file($install_file);
					$install_file = str_replace('.game', '', $install_file);

					$this->file_functions->write_file($install_file, str_replace($search, $replace, $file_contents));

					@include($install_file);

					if (!empty($title))
					{
						$convert_install_file = true;
						$data = array(
							'game_name'			=> $title,
							'game_desc'			=> (isset($description)) ? $description : '',
							'game_control'		=> UNKNOWN_GAME,
							'game_control_desc'	=> '',
							'game_width'		=> (isset($game_width)) ? $game_width : 640,
							'game_height'		=> (isset($game_height)) ? $game_height : 480,
							'game_scoretype'	=> SCORETYPE_HIGH,
							'game_type'			=> V3ARCADE_GAME
						);
					}
				}
				else
				{
					if (strpos($file_contents, '$game_data') !== false)
					{
						$update = false;
						$convert_install_file = true;
					}
					else if (strpos($file_contents, '$ibp_config') === false)
					{
						if (strpos($file_contents, '$config') === false)
						{
							$update = false;
						}

						// We have to read the ibpro config file contents and replace the $config variable
						// or there will be problems since we will have overwritten the phpbb config variable
						if ($update)
						{
							$ibp_config = str_replace('$config', '$ibp_config', $file_contents);
							$this->file_functions->write_file($install_file, $ibp_config);
						}
					}

					if ($update)
					{
						// Since we now have a nice config file lets include it
						@include($install_file);

						// Remove extra spaces from array strings
						foreach ($ibp_config as $key => $value)
						{
							if (is_string($value))
							{
								$ibp_config[$key] = trim($value);
							}
						}

						// $ibp_config['gname'] -> $game_filename

						$convert_install_file = true;
						$gkeys = strtolower($ibp_config['gkeys']);

						$data = array(
							'game_name'			=> $ibp_config['gtitle'],
							'game_desc'			=> $ibp_config['gwords'] . ((!empty($ibp_config['object']) && $ibp_config['object'] != '1' && $ibp_config['gwords'] != $ibp_config['object']) ? ' ' . $ibp_config['object'] : ''),
							'game_control'		=> (strpos($gkeys, 'mouse') !== false) ? GAME_CONTROL_MOUSE : UNKNOWN_GAME,
							'game_control_desc'	=> ($gkeys && $gkeys != 'mouse' && $gkeys != 'mouse.') ? $ibp_config['gkeys'] : '',
							'game_width'		=> $ibp_config['gwidth'],
							'game_height'		=> $ibp_config['gheight'],
							'game_scoretype'	=> (!isset($ibp_config['highscore_type']) || $ibp_config['highscore_type'] == 'high') ? SCORETYPE_HIGH : SCORETYPE_LOW
						);

						$gamedata_path = $this->root_path . 'arcade/gamedata/' . $game_filename;

						// Find game type
						if (file_exists($gamedata_path . '/v32game.txt') || file_exists($gamedata_path . '/v3game.txt'))
						{
							// We have found a v3x game
							$data['game_type'] = IBPROV3_GAME;
						}
						else if (file_exists($gamedata_path . '/'. $game_filename . '.txt'))
						{
							// We have found an arcadelib game
							$data['game_type'] = ARCADELIB_GAME;
						}
						else
						{
							$data['game_type'] = IBPRO_GAME;
						}
					}
				}

				unset($file_contents);
			}

			if (!$update || !$convert_install_file)
			{
				if (empty($game_data))
				{
					@include($install_file);
				}

				if (!empty($game_data) && !$this->version_check($install_file) || !$this->require_var($game_data))
				{
					$data = $game_data;
					$update = $convert_install_file = true;
				}
			}

			// Here is where we create the new install file and place it in the correct location
			if ($update && $convert_install_file)
			{
				if (!$this->update(false, $data, $install_file))
				{
					$convert_install_file = false;
				}
			}

			return ($convert_install_file) ? $game_filename : false;
		}

		return false;
	}

	public function create(&$data)
	{
		static $_cache_default_install_file;

		if (empty($_cache_default_install_file))
		{
			$_cache_default_install_file = file_get_contents($this->arcade->ext_path() . 'inc/game/default_install_file.' . $this->php_ext);
		}

		$data['game_name']			= utf8_normalize_nfc((!empty($data['game_name'])) ? $data['game_name'] : 'Empty');
		$data['game_desc']			= utf8_normalize_nfc((!empty($data['game_desc'])) ? $data['game_desc'] : '');
		$data['game_control']		= (!empty($data['game_control'])) ? $data['game_control'] : '';
		$data['game_control_desc']	= utf8_normalize_nfc((!empty($data['game_control_desc'])) ? $data['game_control_desc'] : '');
		$data['game_image']			= (!empty($data['game_image'])) ? $data['game_image'] : '';
		$data['game_swf']			= (!empty($data['game_swf'])) ? $data['game_swf'] : 'Empty';
		$data['game_scorevar']		= (!empty($data['game_scorevar'])) ? $data['game_scorevar'] : 'Empty';
		$data['game_type']			= (!empty($data['game_type'])) ? $data['game_type'] : UNKNOWN_GAME;
		$data['game_width']			= (!empty($data['game_width'])) ? $data['game_width'] : ARCADE_FLASH_WIDTH;
		$data['game_height']		= (!empty($data['game_height'])) ? $data['game_height'] : ARCADE_FLASH_HEIGHT;
		$data['game_scoretype']		= (!empty($data['game_scoretype'])) ? $data['game_scoretype'] : SCORETYPE_HIGH;

		$game_type					= $this->arcade->game()->type($data['game_type']);
		$data['game_type']			= constant($game_type);

		$data['game_width']			= (int) $data['game_width'];
		$data['game_height']		= (int) $data['game_height'];

		if ($data['game_width'] <= 0 || $data['game_height'] <=0)
		{
			$data['game_width']		= ARCADE_FLASH_WIDTH;
			$data['game_height']	= ARCADE_FLASH_HEIGHT;
		}

		switch ($data['game_control'])
		{
			case GAME_CONTROL_KEYBOARD_MOUSE:
				$game_control = 'GAME_CONTROL_KEYBOARD_MOUSE';
			break;

			case GAME_CONTROL_KEYBOARD:
				$game_control = 'GAME_CONTROL_KEYBOARD';
			break;

			case GAME_CONTROL_MOUSE:
				$game_control = 'GAME_CONTROL_MOUSE';
			break;

			default:
				$game_control = 'UNKNOWN_GAME';
			break;
		}

		$data['game_control'] = constant($game_control);

		switch ($data['game_scoretype'])
		{
			case SCORETYPE_HIGH:
				$game_scoretype = 'SCORETYPE_HIGH';
			break;

			case SCORETYPE_LOW:
				$game_scoretype = 'SCORETYPE_LOW';
			break;

			default:
				$game_scoretype = 'SCORETYPE_HIGH';
			break;
		}

		$data['game_scoretype'] = constant($game_scoretype);

		$game_image = '$game_file . ' . "'.gif'";
		$game_scorevar = '$game_file';

		if (!empty($data['game_swf']))
		{
			$game_folder = $this->file_functions->remove_extension($data['game_swf']);

			if (!empty($data['game_image']))
			{
				if ($this->file_functions->remove_extension($data['game_image']) == $game_folder)
				{
					$game_image = '$' . "game_file . '" . strrchr($data['game_image'], '.') . "'";
				}
				else
				{
					$game_image = "'" . $this->arcade->check_addslashes($data['game_image']) . "'";
				}
			}

			if (!empty($data['game_scorevar']))
			{
				if ($this->file_functions->remove_extension($data['game_scorevar']) != $game_folder)
				{
					$game_scorevar = "'" . $this->arcade->check_addslashes($data['game_scorevar']) . "'";
				}
			}
		}

		$search = array('{GAME_NAME}', "'{GAME_SCOREVAR}'", '{GAME_DESC}', "'{GAME_CONTROL}'", '{GAME_CONTROL_DESC}', "'{GAME_IMAGE}'", "'{GAME_TYPE}'", "'{GAME_WIDTH}'", "'{GAME_HEIGHT}'", "'{GAME_SCORETYPE}'");
		$replace = array($this->arcade->check_addslashes($data['game_name']), $game_scorevar, $this->arcade->check_addslashes($data['game_desc']), $game_control, $this->arcade->check_addslashes($data['game_control_desc']), $game_image, $game_type, $data['game_width'], $data['game_height'], $game_scoretype);

		return str_replace($search, $replace, $_cache_default_install_file);
	}

	public function update($game_id, $data = false, $install_file = false)
	{
		if ($game_id && !$data)
		{
			$data = $this->arcade->get()->game_data($game_id);
		}

		if (empty($data) || (!$install_file && empty($data['game_swf'])))
		{
			return false;
		}

		if (!$install_file)
		{
			$install_file = $this->arcade->set_path($data['game_swf'], 'install');
		}

		$return = $this->file_functions->write_file($install_file, $this->create($data));

		if (!$return)
		{
			if (!$game_id)
			{
				if (empty($data['game_id']))
				{
					if (!empty($data['game_name']))
					{
						$sql = "SELECT game_id
								FROM " . ARCADE_GAMES_TABLE . "
								WHERE game_name_clean = '" . $this->db->sql_escape(utf8_clean_string($data['game_name'])) . "'";
						$result = $this->db->sql_query($sql);
						$game_id = (int) $this->db->sql_fetchfield('game_id');
						$this->db->sql_freeresult($result);
					}
				}
				else
				{
					$game_id = $data['game_id'];
				}
			}

			if ($game_id && !empty($data['game_name']))
			{
				$this->arcade->add_log('critical', $game_id, 'LOG_ARCADE_GAME_INSTALL_FILE_' . ((file_exists($install_file)) ? 'UNWRITABLE' : 'NOT_FOUND'), $data['game_name'], $this->arcade->root_key($install_file));
			}
		}

		return $return;
	}

	public function check_db_and_file_content($db_info, $file_info)
	{
		foreach ($file_info as $key => $value)
		{
			if (isset($db_info[$key]))
			{
				if ($this->arcade->check_addslashes($db_info[$key]) != $this->arcade->check_addslashes($value))
				{
					return false;
				}
			}
		}

		return true;
	}

	public function require_var($game_data, $install_file = false)
	{
		if (!$game_data && $install_file)
		{
			@include($install_file);
		}

		$r = true;
		$require_var = array(
			'game_name', 'game_desc', 'game_control', 'game_control_desc', 'game_image', 'game_swf',
			'game_scorevar', 'game_type', 'game_width', 'game_height', 'game_scoretype'
		);

		foreach ($require_var as $var)
		{
			if (!isset($game_data[$var]))
			{
				$r = false;
				break;
			}
		}

		return $r;
	}

	public function version_check($install_file)
	{
		$r = 0;

		if ($install_file && file_exists($install_file))
		{
			$version = file_get_contents($install_file, null, null, 47, 5);

			if (version_compare($version, '5.0.0', '>='))
			{
				$r = true;
			}
		}

		return $r;
	}
}
