<?php
/**
*
* @package phpBB Arcade
* @version $Id$
* @author 2011-2017 KillBill - killbill@jatek-vilag.com
* @copyright (c) 2014-2017 https://jv-arcade.com/ - support@jv-arcade.com
* @license http://opensource.org/licenses/gpl-2.0.php GNU General Public License v2
*
*/

namespace jv\arcade\inc;

class menu
{
	protected $db, $cache, $user, $auth, $config, $template, $arcade, $arcade_auth, $arcade_config, $php_ext;

	public function __construct($db, $cache, $user, $auth, $config, $template, $arcade, $arcade_auth, $arcade_config, $php_ext)
	{
		$this->db = $db;
		$this->cache = $cache;
		$this->user = $user;
		$this->auth = $auth;
		$this->config = $config;
		$this->template = $template;
		$this->arcade = $arcade;
		$this->arcade_auth = $arcade_auth;
		$this->arcade_config = $arcade_config;
		$this->php_ext = $php_ext;
	}

	public function main()
	{
		$main_menu = $this->obtain_menu();

		if (sizeof($main_menu))
		{
			$pd_id = 0;
			$page = $this->arcade->page();

			foreach ($main_menu['pd'] as $p_id => $m_rows)
			{
				if ($pd_id)
				{
					break;
				}

				foreach ($m_rows as $m_page => $params)
				{
					$m_page = strtolower(basename($m_page, '.' . $this->php_ext));

					if ($page == $m_page || $m_page == 'onclick')
					{
						$found = false;
						foreach ($params as $param)
						{
							if ($this->arcade->page(false, true))
							{
								if ($param)
								{
									$param = str_replace(array('{USER_ID}', '&amp;', 'sid={SID}'), array($this->user->data['user_id'], '&', ''), $param);

									if ($m_page == 'onclick')
									{
										if ($param && strpos($param, $this->arcade->page(false)) !== false)
										{
											$found = true;
											break;
										}
									}
									else if ($param === $this->arcade->page(false, true))
									{
										$found = true;
										break;
									}
								}
							}
							else if (!$param)
							{
								$found = true;
								break;
							}
						}

						if ($found)
						{
							$pd_id = $p_id;
							break;
						}
					}
				}
			}

			unset($main_menu['pd']);

			foreach ($main_menu as $parent_id => $menu)
			{
				if ((!$menu['menu_display'] && !$menu['quick_link']) || !$this->auth($menu['auth']) || (!$this->arcade->access() && ($menu['menu_page'] == 'arcade' || strpos($menu['auth'], 'arcade') !== false)))
				{
					unset($main_menu[$parent_id]);
					continue;
				}

				$width = utf8_strlen($this->arcade->lang_value($menu['menu_name']));

				if (!empty($menu['sub']))
				{
					foreach ($menu['sub'] as $sub_key => $sub_menu)
					{
						if ((!$sub_menu['menu_display'] && !$sub_menu['quick_link']) || !$this->auth($sub_menu['auth']))
						{
							unset($menu['sub'][$sub_key]);
							continue;
						}

						$max_width = utf8_strlen($this->arcade->lang_value($sub_menu['menu_name']));

						if ($width < $max_width)
						{
							$width = $max_width;
						}
					}
				}

				$tkey	= $menu['menu_name'] . '_EXPLAIN';
				$title	= $this->arcade->lang_value($tkey, true);

				if (!$title)
				{
					if (substr($menu['menu_name'], 0, 7) != 'ARCADE_')
					{
						$title = $this->arcade->lang_value('ARCADE_' . $tkey, true);
					}
				}

				$extra_ary	= array(
					'S_ACTIVE_TAB'	=> ($parent_id == $pd_id) ? true : false,
					'S_SIMPLE_TAB'	=> (empty($menu['sub'])) ? true : false,

					'WIDTH'			=> intval(($width * 7) + 16),
					'TITLE'			=> $title
				);

				$this->data($menu, 'arcade_menu', $extra_ary);

				if (!empty($menu['sub']))
				{
					// sub menu
					foreach ($menu['sub'] as $sub_menu)
					{

						if ((!$sub_menu['menu_display'] && !$sub_menu['quick_link']) || !$this->auth($sub_menu['auth']))
						{
							continue;
						}

						$extra_ary = array(
							'S_ACTIVE' => ($sub_menu['menu_page'] && !empty($sub_menu['sub'])) ? true : false
						);

						$this->data($sub_menu, 'arcade_menu.submenu', $extra_ary);

						// sub sub menu
						if (!empty($sub_menu['sub']))
						{
							foreach ($sub_menu['sub'] as $subsub_menu)
							{
								if ((!$subsub_menu['menu_display'] && !$subsub_menu['quick_link']) || !$this->auth($subsub_menu['auth']))
								{
									continue;
								}

								$this->data($subsub_menu, 'arcade_menu.submenu.subsubmenu');
							}
						}
					}
				}
			}

			$this->template->assign_vars(array(
				'S_DISPLAY_ARCADE_MENU'	=> true,
				'S_ARCADE_MENU_SEP'		=> (in_array($this->arcade_config['menu_theme'], array(2, 3))) ? true : false,
				'S_ARCADE_MENU_SQUARE'	=> (in_array($this->arcade_config['menu_theme'], array(0, 2))) ? true : false,

				'ARCADE_MENU_POSITION'	=> $this->arcade->position($this->arcade_config['menu_position'])
			));
		}
	}

	public function data($data, $tpl = false, $extra_ary = array())
	{
		if ($tpl)
		{
			$onclick = '';

			if ($data['menu_page'])
			{
				$sid = (strpos($data['params'], 'sid={SID}') !== false) ? $this->user->session_id : false;
				$params = str_replace(array('{USER_ID}', 'sid={SID}'), array($this->user->data['user_id'], ''), $data['params']);

				if ($data['menu_page'] == 'onclick')
				{
					$onclick = ' onclick="' . $params . '"';
				}
				else
				{
					$url = $this->arcade->url($params, $data['menu_page'], $sid, ($data['menu_top'] == 1) ? true : $data['menu_top']);
					$onclick = ' onclick="' . (($data['new_window']) ? "window.open('" . $url . "')" : "window.location.href = '" . $url . "'") . ';"';
				}
			}

			$array = array(
				'S_INACTIVE'	=> ($onclick == '') ? true : false,

				'NAME'			=> $this->arcade->lang_value($data['menu_name']),
				'ONCLICK'		=> $onclick
			);

			if (sizeof($extra_ary))
			{
				$array = array_merge($array, $extra_ary);
			}

			if ($data['menu_display'])
			{
				$this->template->assign_block_vars($tpl, $array);
			}

			if ($data['quick_link'])
			{
				$this->template->assign_block_vars('arcade_quick_link', array(
					'LINK'	=> $url,
					'NAME'	=> $this->arcade->lang_value($data['menu_name'])
				));
			}
		}
		else
		{
			return array(
				'menu_display'	=> $data['menu_display'],
				'menu_name'		=> $data['menu_name'],
				'menu_page'		=> $data['menu_page'],
				'menu_top'		=> $data['menu_top'],
				'new_window'	=> $data['new_window'],
				'quick_link'	=> $data['quick_link'],
				'params'		=> $data['params'],
				'auth'			=> $data['auth'],
				'left_id'		=> $data['left_id'],
				'right_id'		=> $data['right_id']
			);
		}
	}

	private function auth($menu_auth)
	{
		$menu_auth = trim($menu_auth);

		// Generally allowed to access menu if menu_auth is empty
		if (!$menu_auth)
		{
			return true;
		}

		// With the code below we make sure only those elements get eval'd we really want to be checked
		preg_match_all('/(?:
			"[^"\\\\]*(?:\\\\.[^"\\\\]*)*"		|
			\'[^\'\\\\]*(?:\\\\.[^\'\\\\]*)*\'	|
			[(),]								|
			[^\s(),]+)/x', $menu_auth, $match);

		// Valid tokens for auth and their replacements
		$valid_tokens = array(
			'pa:acl_([a-z0-9_]+)'		=> '(int) $this->auth->acl_get(\'\\1\')',
			'pa:aclc_([a-z0-9_]+)'		=> '(int) $this->arcade_auth->acl_getc_global(\'\\1\')',
			'pa:aclf_([a-z0-9_]+)'		=> '(int) $this->auth->acl_getf_global(\'\\1\')',
			'pa:acfg_([a-z0-9_]+)'		=> '!empty($this->arcade_config[\'\\1\'])',
			'pa:!acfg_([a-z0-9_]+)'		=> 'empty($this->arcade_config[\'\\1\'])',
			'pa:cfg_([a-z0-9_]+)'		=> '!empty($this->config[\'\\1\'])',
			'pa:!cfg_([a-z0-9_]+)'		=> 'empty($this->config[\'\\1\'])',
			'pa:user_([a-z0-9_]+)'		=> '!empty($this->user->data[\'\\1\'])',
			'pa:!user_([a-z0-9_]+)'		=> 'empty($this->user->data[\'\\1\'])',
			'pa:points_([a-z0-9_]+)'	=> '!empty($this->arcade->points()->data[\'\\1\'])',
			'pa:portal_([a-z0-9_]+)'	=> '!empty($this->arcade->portal()->data[\'\\1\'])',
			'pa:shout_([a-z0-9_]+)'		=> '!empty($this->arcade->shout()->data[\'\\1\'])',
			'pa:ext_([a-zA-Z0-9_/]+)'	=> '!empty($this->arcade->ext_enable(\'\\1\'))'
		);

		$tokens = $match[0];
		for ($i = 0, $size = sizeof($tokens); $i < $size; $i++)
		{
			$token = &$tokens[$i];

			switch ($token)
			{
				case ')':
				case '(':
				case '&&':
				case '||':
				case ',':
				break;

				default:
					if (!preg_match('#(?:' . str_replace('pa:', '', implode(array_keys($valid_tokens), ')|(?:')) . ')#', $token))
					{
						$token = '';
					}
					else
					{
						$token = 'pa:' . $token;
					}
				break;
			}
		}

		$is_auth = false;

		eval('$is_auth = (int) (' . preg_replace(array_map(function($value) {return '#' . $value . '#';}, array_keys($valid_tokens)), array_values($valid_tokens), implode(' ', $tokens)) . ');');

		return $is_auth;
	}

	public function obtain_menu($no_cache = false)
	{
		if ($no_cache || ($main_menu = $this->cache->get('_arcade_menu')) === false)
		{
			$data = $main_menu = $pd = array();

			$sql = 'SELECT menu_id, menu_name, menu_display, menu_page, new_window, quick_link, parent_id, params, menu_top, auth, left_id, right_id
					FROM ' . ARCADE_MENU_TABLE . '
					ORDER BY left_id ASC';
			$result = $this->db->sql_query($sql);
			while ($row = $this->db->sql_fetchrow($result))
			{
				$data[$row['menu_id']] = array(
						'parent_id'		=> $row['parent_id'],
						'menu_name'		=> $row['menu_name'],
						'menu_display'	=> $row['menu_display'],
						'menu_page'		=> $row['menu_page'],
						'menu_top'		=> $row['menu_top'],
						'new_window'	=> $row['new_window'],
						'quick_link'	=> $row['quick_link'],
						'params'		=> $row['params'],
						'auth'			=> $row['auth'],
						'left_id'		=> $row['left_id'],
						'right_id'		=> $row['right_id']
				);
			}
			$this->db->sql_freeresult($result);

			foreach ($data as $menu_id => $row)
			{
				if (!$row['parent_id'])
				{
					$main_menu[$menu_id] = $this->data($row);
				}
				else
				{
					$main_menu[$row['parent_id']]['sub'][$menu_id] = $this->data($row);

					foreach ($data as $mid => $sub)
					{
						if ($row['left_id'] < $sub['left_id'] && $row['right_id'] > $sub['right_id'])
						{
							$main_menu[$row['parent_id']]['sub'][$menu_id]['sub'][$mid] = $this->data($sub);
						}
					}
				}
			}

			foreach ($main_menu as $parent_id => $menu)
			{
				if (!isset($menu['menu_display']) || (!$no_cache && !$menu['menu_display'] && !$menu['quick_link']))
				{
					unset($main_menu[$parent_id]);
					continue;
				}

				if ($menu['menu_page'])
				{
					$pd[$parent_id][$menu['menu_page']][] = $menu['params'];
				}

				if (!empty($menu['sub']))
				{
					foreach ($menu['sub'] as $sub_parent_id => $sub_menu)
					{
						if (!$no_cache && !$sub_menu['menu_display'] && !$sub_menu['quick_link'])
						{
							unset($menu['sub'][$sub_parent_id]);
							continue;
						}

						if ($sub_menu['menu_page'])
						{
							$pd[$parent_id][$sub_menu['menu_page']][] = $sub_menu['params'];
						}

						if (!empty($sub_menu['sub']))
						{
							foreach ($sub_menu['sub'] as $subsub_parent_id => $subsub_menu)
							{
								if (!$no_cache && !$subsub_menu['menu_display'] && !$subsub_menu['quick_link'])
								{
									unset($sub_menu['sub'][$subsub_parent_id]);
									continue;
								}

								if ($subsub_menu['menu_page'])
								{
									$pd[$parent_id][$subsub_menu['menu_page']][] = $subsub_menu['params'];
								}
							}
						}
					}
				}
			}

			$main_menu['pd'] = (!empty($pd)) ? $pd : array();
			unset($pd);

			if (!$no_cache)
			{
				$this->cache->put('_arcade_menu', $main_menu);
			}
		}

		return $main_menu;
	}
}
